import falcon
import raven


class Sentry:
    """Simple Sentry integration plugin

        config = {'sentry_dsn': 'some@sentry.io'}
        app = falcon.API()
        sentry = Sentry(config['sentry_dsn'])
        sentry.register_app(sentry)

    You can get access to this globally so that you can capture exceptions without it having to get
    all the way up to the Falcon layer again.  Any exception that is caught by the falcon error
    handler will be sent to sentry (except HTTPError's and HTTPStatus's).

        sentry = Sentry()

        def create_app(config):
            app = falcon.API()

            sentry.set_dsn(config.sentry_dsn)
            sentry.register(app)
            return app


        class SomeResource:

            def on_get(self, req, resp):
                try:
                    some_failure()
                except Exception as e:
                    sentry.captureException(e)
                    pass
    """
    def __init__(self, dsn=None):
        self.set_dsn(dsn)

    def set_dsn(self, dsn):
        self.dsn = dsn
        self.client = raven.Client(dsn)

    def register(self, app):
        app.add_error_handler(Exception, self.handle)

    def handle(self, ex, req, resp, params):
        raisable = (falcon.http_error.HTTPError, falcon.http_status.HTTPStatus)

        if isinstance(ex, raisable):
            raise ex
        elif self.dsn:
            self.client.captureException(ex)
            raise falcon.HTTPInternalServerError()
        else:
            raise



from sqlalchemy.orm import (
    scoped_session,
    sessionmaker,
)

import falcon_helpers.sqla.db as db


class SQLAlchemySessionMiddleware:
    def __init__(self, session):
        # self.session = session
        pass

    def process_resource(self, req, resp, resource, params):
        resource.session = db.session

    def process_response(self, req, resp, resource, req_succeeded):
        if not hasattr(resource, 'session'):
            return

        try:
            if not req_succeeded:
                db.session.rollback()
            else:
                db.session.commit()
        except Exception as e:
            db.session.remove()
            raise



import configparser


class ConfigurationError(Exception):
    pass


class Config(dict):
    """A basic Configuration type

    This configuration class can use subscriptable syntax `['item']` or dotted syntaxt `config.item`
    """

    def __getattr__(self, item):
        return self.__getitem__(item)

    def __setattr__(self, item, value):
        return self.__setitem__(item, value)

    def __delattr__(self, item):
        return self.__delitem__(item)

    def __getitem__(self, item):
        try:
            return super().__getitem__(item)
        except KeyError as e:
            raise ConfigurationError(f'Unable to find configuration value for key {item}')

    @classmethod
    def _parsed_to_dict(cls, config):
        return cls([
            (s, cls(config.items(s)))
            for s in config.sections()
        ])

    @classmethod
    def from_inis(cls, *fpaths):
        """Read the configuration for ini files

        *fpaths is passed straight to `ConfigParser.read` so it can be multiple configuration files
        and it will read them in order.
        """
        config = configparser.ConfigParser()
        config.read(fpaths)

        return cls._parsed_to_dict(config)

